/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MainViewAS.as 21081 2011-03-31 08:55:51Z renaultgu $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.admin.webservicesconsole.view {
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.events.Event;
import flash.events.MouseEvent;

import mx.collections.ArrayCollection;
import mx.containers.Panel;
import mx.controls.ProgressBar;
import mx.controls.ProgressBarMode;
import mx.controls.TextArea;
import mx.controls.TextInput;
import mx.events.ListEvent;

import org.ow2.jonas.admin.webservicesconsole.event.WebServicesConsoleEvent;
import org.ow2.jonas.admin.webservicesconsole.model.WebServicesConsoleModel;

/**
 * Main view of the webService console
 *
 * @author Guillaume Renault
 */
public class MainViewAS extends Panel {

    //=========================================================================
    //
    //  Variables
    //
    //=========================================================================

    //-------------------------------------------------------------------------
    //  Public
    //-------------------------------------------------------------------------
    /**
     * Whether the get operation button is enabled or not.
     * @default <code>true</code>
     */
    [Bindable]
    public var isGetOperationsButtonEnabled:Boolean = true;

    /**
     * Whether the send soap request button is enabled or not.
     * @default <code>false</code>
     */
    [Bindable]
    public var isSendSoapRequestButtonEnabled:Boolean = false;

    //-------------------------------------------------------------------------
    //  Private binding
    //-------------------------------------------------------------------------

    /**
     * The soap response
     */
    private var m_SoapResponse:String = null;

    /**
     * List of all available operations
     */
    [ArrayElementType("String")]
    private var m_Operations:ArrayCollection = null;

    /**
     * The soap request
     */
    private var m_SoapRequestString:String = null;

    //-------------------------------------------------------------------------
    //  Private 
    //-------------------------------------------------------------------------

    /**
     * Currently selected operation.
     */
    private var m_SelectedOperation:String = "";

    /**
     * reference of the progress bar.
     */
    private var m_ProgressBar:ProgressBar = null;

    //=========================================================================
    //
    //  Constructor
    //
    //=========================================================================
    public function MainViewAS() {
    }

    //=========================================================================
    //
    //  Methods
    //
    //=========================================================================

    //-------------------------------------------------------------------------
    //  Handlers
    //-------------------------------------------------------------------------

    /**
     * Get all the available operations for the given WebService.
     * @param event
     * @param p_UrlTI the TextInput container that contains the URL.
     */
    protected function handleGetWSOperationsButton(event:MouseEvent, p_UrlTI:TextInput):void {

        // set to null in case of the same answer
        this.m_SoapRequestString = "";
        WebServicesConsoleModel.getInstance().soapRequestSkeleton = null;
        this.soapResponse = "";
        WebServicesConsoleModel.getInstance().soapResponse = null;
        // remove all operations from the list while waiting for the call answer
        if (null != this.operations) {
            this.m_Operations.removeAll();
        }

        // show the progress bar
        this.handleShowProgressBar();
        this.isGetOperationsButtonEnabled = false;
        this.isSendSoapRequestButtonEnabled = false;

        var webServiceCallEvent:WebServicesConsoleEvent = new WebServicesConsoleEvent(WebServicesConsoleEvent.GET_AVAILABLE_OPERATIONS + WebServicesConsoleModel.getInstance().componentID);
        webServiceCallEvent.webserviceURL = p_UrlTI.text;

        CairngormEventDispatcher.getInstance().dispatchEvent(webServiceCallEvent);
    }


    /**
     * Handle when the user selects an operation from the available list.
     * @param event
     * @param p_UrlTI the TextInput container that contains the URL.
     */
    protected function handleOperationSelected(event:ListEvent, p_UrlTI:TextInput):void {

        // set to null in case of the same answer
        this.m_SoapRequestString = "";
        WebServicesConsoleModel.getInstance().soapRequestSkeleton = null;
        this.soapResponse = "";
        WebServicesConsoleModel.getInstance().soapResponse = null;

        // show the progress bar
        this.handleShowProgressBar();
        this.isGetOperationsButtonEnabled = false;
        this.isSendSoapRequestButtonEnabled = false;

        // keep the selected operation
        this.m_SelectedOperation = String(this.operations.getItemAt(event.rowIndex));

        var webServiceCallEvent:WebServicesConsoleEvent = new WebServicesConsoleEvent(WebServicesConsoleEvent.GET_SOAP_REQUEST + WebServicesConsoleModel.getInstance().componentID);
        webServiceCallEvent.webserviceURL = p_UrlTI.text;
        webServiceCallEvent.webserviceOperation = this.m_SelectedOperation;

        CairngormEventDispatcher.getInstance().dispatchEvent(webServiceCallEvent);
    }

    /**
     * Send the soap request to the webservice.
     * @param event
     * @param p_RequestTA the TextArea container that contains the request to send.
     * @param p_UrlTI the TextInput container that contains the URL.
     */
    protected function handleSendWSRequestButton(event:MouseEvent, p_RequestTA:TextArea, p_UrlTI:TextInput):void {

        // set to null in case of the same answer
        this.soapResponse = "";
        WebServicesConsoleModel.getInstance().soapResponse = null;

        // show the progress bar
        this.handleShowProgressBar();
        this.isGetOperationsButtonEnabled = false;
        this.isSendSoapRequestButtonEnabled = false;

        var webServiceCallEvent:WebServicesConsoleEvent = new WebServicesConsoleEvent(WebServicesConsoleEvent.SEND_SOAP_REQUEST + WebServicesConsoleModel.getInstance().componentID);
        webServiceCallEvent.soapRequest = p_RequestTA.text;
        webServiceCallEvent.webserviceOperation = this.m_SelectedOperation;
        webServiceCallEvent.webserviceURL = p_UrlTI.text;

        CairngormEventDispatcher.getInstance().dispatchEvent(webServiceCallEvent);
    }

    /**
     * Keep a reference of the progress bar once created.
     *
     * @param event
     * @param p_ProgressBar
     */
    protected function handleProgressBarCreated(event:Event, p_ProgressBar:ProgressBar):void {
        this.m_ProgressBar = p_ProgressBar;
    }

    //-------------------------------------------------------------------------
    //  Getter/Setter
    //-------------------------------------------------------------------------

    [Bindable]
    public function set soapRequestString(p_SoapRequestString:String):void {
        this.m_SoapRequestString = p_SoapRequestString;
        this.handleHideProgressBar();
        this.isGetOperationsButtonEnabled = true;
        this.isSendSoapRequestButtonEnabled = true;
    }

    public function get soapRequestString():String {
        return this.m_SoapRequestString;
    }

    [Bindable]
    public function set soapResponse(p_SoapResponse:String):void {
        this.m_SoapResponse = p_SoapResponse;
        this.handleHideProgressBar();
        this.isGetOperationsButtonEnabled = true;
        this.isSendSoapRequestButtonEnabled = true;
    }

    public function get soapResponse():String {
        return this.m_SoapResponse;
    }

    [Bindable]
    public function set operations(p_Operations:ArrayCollection):void {
        this.m_Operations = p_Operations;
        this.handleHideProgressBar();
        this.isGetOperationsButtonEnabled = true;
        this.isSendSoapRequestButtonEnabled = false;
    }

    public function get operations():ArrayCollection {
        return this.m_Operations;
    }

    //-------------------------------------------------------------------------
    //  Private functions
    //-------------------------------------------------------------------------


    private function handleShowProgressBar():void {
        if (null != this.m_ProgressBar) {
            this.m_ProgressBar.visible = true;
            this.m_ProgressBar.indeterminate = true;
            this.m_ProgressBar.setStyle("themeColor", 0xEE1122);
            this.m_ProgressBar.includeInLayout = true;
        }
    }

    private function handleHideProgressBar():void {
        if (null != this.m_ProgressBar) {
            this.m_ProgressBar.indeterminate = false;
            this.m_ProgressBar.setStyle("themeColor", 0x66cc00);
            this.m_ProgressBar.mode = ProgressBarMode.MANUAL;
            this.m_ProgressBar.setProgress(100, 100);
            this.m_ProgressBar.mode = ProgressBarMode.EVENT;
        }
    }

}
}
